/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import netscape.ldap.*;
import netscape.ldap.controls.*;

import util.*;

class VListEntryModel extends AbstractListModel {
	VListEntryModel( LDAPConnection ldc, String base,
					 int scope, String filter ) {
		_ldc = (LDAPConnection)ldc.clone();
		_base = base;
		_scope = scope;
		_filter = filter;
		_host = ldc.getHost();
		_port = ldc.getPort();
		_authDN = ldc.getAuthenticationDN();
		_authPassword = ldc.getAuthenticationPassword();
	}

	/**
	 * Called by JList to get virtual list size
	 * @return Virtual list size
	 */
    public int getSize() {
		Debug.println(9, "VListEntryModel.getSize()");
		if ( !_initialized ) {
			_initialized = true;
			_pageControls = new LDAPControl[2];
			// Paged results also require a sort control
			LDAPSortKey[] keys = new LDAPSortKey[5];
			int i = 0;
			keys[i++] = new LDAPSortKey( "sn" );
			keys[i++] = new LDAPSortKey( "givenname" );
			keys[i++] = new LDAPSortKey( "cn" );
			keys[i++] = new LDAPSortKey( "ou" );
			keys[i++] = new LDAPSortKey( "o" );
			_pageControls[0] =
                new LDAPSortControl( keys, true );
			// Do an initial search to get the virtual list
            // size. Keep one page before and one page after
            // the start
			_beforeCount = _pageSize;
			_afterCount = _pageSize;
			// Create the initial paged results control
			LDAPVirtualListControl cont =
				new LDAPVirtualListControl( "A",
											_beforeCount,
											_afterCount );
			_pageControls[1] = cont;
			_vlc = (LDAPVirtualListControl)_pageControls[1];
			getPage( 0 );
		}
		Debug.println(9, "VListEntryModel.getSize()=" + _size);
		return _size;
	}

    public Object getElementAt(int index) {
		Debug.println(9, "VListEntryModel.getElementAt: need " +
                      " entry " + index );
		if ( _mismatch ) {
			Debug.println(9, "VListEntryModel.getElementAt: " +
                          "mismatch; returning empty entry");
			// Empty entry
			LDAPEntry entry = new LDAPEntry();
			_entries.addElement( entry );
			return entry;
		}
		Debug.println(9, "VListEntryModel.getElementAt: " +
                      "_top=" + _top + " _entries.size()=" +
                      _entries.size());
		if ( (index < _top) ||
             (index >= _top + _entries.size()) ) {
			Debug.println(9, "VListEntryModel.getElementAt: " +
                          "fetching a page starting at " +
                          index );
			if ( !getPage( index ) ) {
				Debug.println(
                    9, "VListEntryModel.getElementAt: " +
                    "first getPage(" + index + ") is null");
				return new LDAPEntry();
			}
			// DS 4.0 may return entries that do not include
            // the requested one! If so, reduce the preceding
            // offset.
			int offset = index - _top;
			if ( offset >= _entries.size() ) {
				_beforeCount = _entries.size() / 3;
				_afterCount = _pageSize * 2 - _beforeCount;
				if ( !getPage( index ) ) {
					Debug.println(
                        9, "VListEntryModel.getElementAt: " +
                        "second getPage(" + index +
                        ") is null");
					return new LDAPEntry();
				}
			}
		}
		int offset = index - _top;
		if ( (offset < 0) || (offset >= _entries.size()) ) {
			Debug.println(9, "VListEntryModel.getElementAt: " +
						  "bogus offset=" + offset);
			// Empty entry
			LDAPEntry entry = new LDAPEntry();
			_entries.addElement( entry );
			return entry;
		}
		else {
			Debug.println(9, "VListEntryModel.getElementAt: " +
						  "success for element=" + offset);
			LDAPEntry entry =
				(LDAPEntry)_entries.elementAt( offset );
			Debug.println(9, "VListEntryModel.getElementAt: " +
						  "object=" + entry);
			return entry;
		}
	}

	/**
	 * Force recalculation of list size
	 */
	public void recalculate() {
		Debug.println(9, "VListEntryModel.recalculate");
		_initialized = false;
		getSize();
	}
	
	private String getNameForEntry( LDAPEntry entry ) {
		LDAPAttribute attr = entry.getAttribute( "cn" );
		if ( attr != null ) {
			Enumeration en = attr.getStringValues();
			if( en.hasMoreElements() ) {
				return (String)en.nextElement();
			}
		}
		String[] rdns =
            LDAPDN.explodeDN( entry.getDN(), true );
		return rdns[0];
	}

	private LDAPVirtualListResponse getEntries() {
		LDAPVirtualListResponse lVLresponse = null;
		if ( _ldc == null ) {
			return lVLresponse;
		}
		// Specify necessary controls for vlv
		LDAPSearchConstraints cons =
			_ldc.getSearchConstraints();
		cons.setMaxResults( 0 );
		LDAPControl[] controls = _pageControls;
		if ( !getReferralsEnabled() ) {
			controls = new LDAPControl[_pageControls.length + 1];
			for( int i = 0; i < _pageControls.length; i++ ) {
				controls[i] = _pageControls[i];
			}
			controls[_pageControls.length] = _manageDSAITControl;
			Debug.println(9, "VListEntryModel.getEntries: " +
                          "referrals not enabled");
		} else {
			Debug.println(9, "VListEntryModel.getEntries: " +
                          "referrals are enabled");
		}
		cons.setServerControls( controls );

		// Empty the buffer
		_entries.removeAllElements();
		// Do a search
		try {
			String[] attrs = { "cn", "objectclass",
                               "numsubordinates" };
			Debug.println(8, "VListEntryModel.getEntries: " +
                          "searching <" +
						  _base + ">, " + _filter );
			LDAPSearchResults result =
				_ldc.search( _base,
							 _scope,
							 _filter,
							 attrs,
							 false,
							 cons );
			int numReadEntries = 0;
			while ( result.hasMoreElements() ) {
				try {
					Debug.println(
                        9, "VListEntryModel.getEntries: " +
						"getting next entry from result" );
					LDAPEntry entry = result.next();
					Debug.println(
                        9, "VListEntryModel.getEntries: " +
                        "adding <" + entry.getDN() + ">" );
					_entries.addElement( entry );
					numReadEntries++;
				} catch ( LDAPReferralException ref ) {
					// This clause only executed if automatic
                    // referral following is off (which it
                    // isn't)
					LDAPUrl refUrls[] = ref.getURLs();
					for( int i = 0; i < refUrls.length; i++ ) {
						Debug.println(
                            "VListEntryModel.getEntries: " +
                            "referral " + refUrls[i].getUrl() );
					}
				}
			}
			int numRequestedEntries =
                _afterCount + _beforeCount;
			// Check if we have a control returned
			LDAPControl[] c = _ldc.getResponseControls();
			if ( c != null ) {
				lVLresponse = null;
                for( int i = 0; i < c.length; i++ ) {
                    if ( c[i] instanceof
                         LDAPVirtualListResponse ) {
                        lVLresponse =
                            (LDAPVirtualListResponse)c[i];
                        break;
                    }
                }
				if ( lVLresponse != null ) {
					Debug.println(
                        8, "VListEntryModel.getEntries: " +
                        "result <" +
                        result.hasMoreElements() +
                        "> total content count=" +
                        lVLresponse.getContentCount());
					if ( lVLresponse.getContentCount() <
						 numRequestedEntries ) {
						numRequestedEntries =
                            lVLresponse.getContentCount();
					}
				} else {
					Debug.println(
                        8, "VListEntryModel.getEntries: " +
                        "VLVResponse is null");
				}
			} else {
				Debug.println(
                    8, "VListEntryModel.getEntries: " +
                    "null response controls");
				// this will typically happen when searching 
                // the front end entries, e.g. cn=config. VLV
                // searches are not supported on those
                // entries. Just assume the number of entries
                // read is the number we requested.
				numRequestedEntries = numReadEntries;
			}
			Debug.println(9, "VListEntryModel.getEntries: " +
                          "requested " + numRequestedEntries +
                          " entries, read " + numReadEntries);
			for (int ii = numReadEntries;
                 ii < numRequestedEntries; ++ii) {
				// Fill the remainder with empty entries
				LDAPEntry entry = new LDAPEntry();
				_entries.addElement( entry );
			}
		} catch ( LDAPException e ) {
			Debug.println( "VListEntryModel.getEntries: " + e +
						   ", searching" );
			if ( _pageControls == null ) {
				return null;
			}
		}
		Debug.println(8, "VListEntryModel.getEntries: " +
                      "Returning " + _entries.size() +
                      " entries for " + _base );
		return lVLresponse;
	}

	// Get a page starting at first (although we may also fetch
	// some preceding entries)
	boolean getPage( int first ) {
		// Get a full buffer, if possible
		int offset = first - _beforeCount;
		if ( offset < 0 ) {
			first -= offset;
		}
		_vlc.setRange( first, _beforeCount, _afterCount );
		Debug.println(9, "VListEntryModel.getPage: Setting " +
                      "requested range to " +
					  first + ", -" + _beforeCount + ", +" +
					  _afterCount );
		return getPage();
	}
	
	// Fetch a buffer
	boolean getPage() {
		// Get the actual entries
		LDAPVirtualListResponse nextCont = getEntries();

		// Check if we have a control returned
		if ( nextCont == null ) {
			_size = _entries.size();
			_vlc.setListSize(_size);
			_selectedIndex = 0;
		} else {
			_selectedIndex = nextCont.getFirstPosition() - 1;
			_top = Math.max( 0,
                             _selectedIndex - _beforeCount );
			Debug.println(9, "VListEntryModel.getPage: " +
                          "_selectedIndex=" + _selectedIndex +
                          " _beforeCount=" + _beforeCount +
						  " _entries.size()=" + _entries.size());
			// Now we know the total size of the virtual list box
			_size = nextCont.getContentCount();
			if ( _size < _entries.size() ) {
				Debug.println( "VListEntryModel.getPage: " +
                               "Reported list size = " +
                               _size );
				_size = _entries.size();
			} else if ( _size > _entries.size() ) {
				// In some cases, e.g. bad referrals, there
                // can be more elements in the vlv control than
                // could actually be retrieved from the
                // directory.  We fill in the list with bogus
                // entries.
				Debug.println( "VListEntryModel.getPage: " +
                               "mismatch " +
							   (_size - _entries.size()) +
                               " entries");
			}
			_vlc.setListSize( _size );
			Debug.println(8, "VListEntryModel.getPage: " +
                          "Virtual window: " +
						   _top + ".." +
                          (_top+_entries.size()-1) +
						   " of " + _size );
			_mismatch = ( (_size > 0) &&
                          (_entries.size() == 0) );
		}
		return true;
	}

	/**
     * Called by application to get the virtual selected index
     *
     * @return The virtual top selection index
     */
    public int getSelectedIndex() {
		return _selectedIndex;
	}

	/**
     * Called by application to get the top of the buffer
     *
     * @return The index of the top of the buffer
     */
    public int getFirstIndex() {
		return _top;
	}

    /**
     * Set the size of the page buffer
     *
     * @param size Size in entries of the buffer
     */
	public void setPageSize( int size ) {
		_pageSize = size;
	}

    /**
     * Get the size of the page buffer
     *
     * @return Size in entries of the buffer
     */
	public int setPageSize() {
		return _pageSize;
	}

    /**
     * Flush the buffers
     */
    public void reset() {
		Debug.println(9, "VListEntryModel.reset");
		if ( _entries.size() > 0 ) {
			Debug.println(9, "VListEntryModel.reset: " +
                          "old first entry=" +
						  _entries.elementAt(0));
            _entries.removeAllElements();
		} else {
			Debug.println(9, "VListEntryModel.reset: " +
                          "already empty");
        }
		_initialized = false;
	}

	/**
	 * Set a parameter for future searches, which determines
     * if the ManagedSAIT control is sent with each search. If
     * referrals are disabled, the control is sent and you will
     * receive the referring entry back.
	 *
	 * @param on <CODE>true</CODE> (the default) if referrals
     * are to be followed
	 */
    public void setReferralsEnabled( boolean on ) {
		Debug.println(
            9, "VListEntryModel.setReferralsEnabled: " + on);
		_followReferrals = on;
	}

	/**
	 * Get the parameter which determines if the
	 * ManagedSAIT control is sent with each search.
	 *
	 * @returns true if referrals are to be followed
	 */
    public boolean getReferralsEnabled() {
		return _followReferrals;
	}

    // Buffer of results read
	private Vector _entries = new Vector();
    private boolean _initialized = false;
    private int _top = 0;
    private int _beforeCount;
	private int _afterCount;
	private int _pageSize = 50;
	private int _selectedIndex = 0;
	private LDAPControl[] _pageControls = null;
	private LDAPVirtualListControl _vlc = null;
	private static LDAPControl _manageDSAITControl =
			new LDAPControl( LDAPControl.MANAGEDSAIT, true, null );
	private boolean _followReferrals = true;
	private boolean _mismatch = false;
	protected int _size = -1;
	private String _base;
	private int _scope;
	private String _filter;
	private String _host;
	private int _port;
	private String _authDN;
	private String _authPassword;
	private LDAPConnection _ldc;
}

