/**
 * Read and decode LDIF file input stream
 */

import java.io.*;


class LdifBufferedReader extends BufferedReader {
	private BufferedReader in;
	private String currentLine = null;
	boolean g_finishReading = false;
	String leftOver = null;

    public LdifBufferedReader(BufferedReader in) {
		super(in);
		this.in = in;
	}

    public LdifBufferedReader(BufferedReader in, int size) {
		super(in, size);
		this.in = in;
    }


	public int read(char cbuf[],int off,int len) throws IOException {
		String block;
		String line;

		if (g_finishReading == true)
			return -1;

		line = readLine();
		if (line == null) {
			g_finishReading = true;
			return -1;
		}

		block = line + "\n";
		while (true) {
			try {
				line = readLine();
				if (line == null) {
					g_finishReading = true;
					break;
				}								
			}
			catch(Exception e) {
				g_finishReading = true;
				break;				
			}

			block += line + "\n";

			if (block.length() >= (off + len))
				break;
		}

		if (block.length() > (off + len)) {
			leftOver = new String(block.substring(off + len,
												  block.length()));
			block.getChars(off, off + len, cbuf, 0);
			return len;
		} else  {
			leftOver = null;
			block.getChars(off, block.length(), cbuf, 0);
			return block.length() - off;
		}	
   	}

 	public int read() throws IOException {
		char ch;
		if (leftOver != null) {
			ch = leftOver.charAt(0);
			leftOver = leftOver.substring(1, leftOver.length());
			return ch;
		}
		else if (currentLine != null) {
			ch = currentLine.charAt(0);
			currentLine = currentLine.substring(1, currentLine.length());
			return ch;
		}
			
		return in.read();
	}
	
	public String readLine() throws IOException {
		synchronized (lock) {
			// if space at beginning, keep reading
			// if end of file, return current buffer

			String line;
			String nextLine;
			int n;		

			if (currentLine == null)
				line = in.readLine();
			else
				line = currentLine;

			if (line == null)
				return null;

			// read next to decide if line if finished.
			for (;;) {
				nextLine = in.readLine();
				if (nextLine == null)
					break;
				if (nextLine.startsWith(" ") == false) {
					break;
				}

				// remove spaces
				line += nextLine.trim();
			}

			currentLine = nextLine;

			// decode base64 if necessary
			if (line.startsWith("#") == false &&
					(n = line.indexOf("::")) != -1) {
				String base64;
				String decodedValue = "";

				base64 =
					new String(line.substring(n+2,
											  line.length()).trim());

				if (Mime.isBase64(base64) == true) {
					byte[] bytes;

					bytes = base64.getBytes();
					try {
						decodedValue =
							new String(Mime.decodeBase64(bytes), "UTF8");
						line = line.substring(0, n+2) + " " + decodedValue;
					}
					catch(Exception e) {
						System.out.println("Decoded base64 is not in UTF8");
					}
				}
			}

			return line;
		}
	}

	boolean insertLine(byte buffer[], int begin, int len) {
		byte line[];
		boolean contain8Bit = false;
		int i;
		String s;

		// get real len vale
		i = 0;
		for (i = 0; i < len; i++) {
			if (buffer[begin+i] == 0) {
				len = i;
				break;
			}
		}

		line = new byte[len];

		for (i = 0; i < len; i++) {
			line[i] = buffer[begin + i];

			if ((line[i] & 0x80) != 0)
				contain8Bit = true;			
		}

		if (contain8Bit == true) {
			try {			
				s = new String(line, "UTF8");
			}
			catch (Exception e) {
				s = new String(line);
			}
		}
		else {
			s = new String(line);
		}

		return insertLine(s);
	}

	String g_textBuffer = "";
	
	boolean insertLine(String s) {
		g_textBuffer += s;
		return true;
	}

	int insertBufferBlock(byte buffer[], int start, int end) {
		int base64Start = 0;
		boolean colon = false;
		boolean doubleColon = false;
		boolean lineContinue = false;			
		int newLineStart = 0;
		int i, k;
		byte line[] = new byte[end - start + 1];
		
		for (k = 0, i = start; i <= end; i++, k++) {
			byte byt;
			line[k] = buffer[i];
			byt = buffer[i];

			if (byt == ' ' && lineContinue == true) {
				k -= 2;
				lineContinue = false;
				continue;
			}
			else if (lineContinue == true) {
				if (base64Start == 0) {
					if (insertLine(line, 0, k) == false)
						return end + 1;

					lineContinue = false;								
				}						
				else {
				    byte base64[];
			    
					base64 = Mime.decodeBase64(line, base64Start,
											   k - base64Start);
					insertLine(line, 0, base64Start);
					if (insertLine(base64, 0, base64.length) == false) {
        				return end + 1;
					}
        			
					base64Start = 0;
				}

				k = 0;
				line[k] = byt;
				newLineStart = i;
			} else if (byt == ' ' && doubleColon == true) {
				base64Start = k + 1;
				doubleColon = false;
			} else if (byt == ':') {
				if (colon == true) {
					doubleColon = true;
					colon = false;
				} else {
					colon = true;
				}
			} else if (buffer[i] == '\n') {				
				lineContinue = true;
			} else if (byt == 0x0d) {
				k--;
				continue;
			} else {
				colon = false;
				doubleColon = false;
				lineContinue = false;			
			}		
		}		
	
		return newLineStart;
	}
}
