/**
 * Encode and write LDIF file output stream
 */

import java.io.*;

class LdifBufferedWriter extends BufferedWriter {
	private BufferedWriter out;
	private String currentLine = null;

	public LdifBufferedWriter(BufferedWriter out) {
		super(out);
		this.out = out;
	}

	public LdifBufferedWriter(BufferedWriter out, int sz) {
		super(out, sz);
		this.out = out;
	}

	public void close() throws IOException {
		writeCurrentLine();
		out.close();
	}

	public void flush() throws IOException {
		writeCurrentLine();
		out.flush();
	}

	public void newLine() throws IOException {
		writeCurrentLine();
		out.newLine();
	}

	public void write(char cbuf[],int off,int len) throws IOException {
		writeLines(new String(cbuf, off, len));
    }

	public void write(int c) throws IOException {
		char ch[];

		ch = new char[1];
		ch[0] = (char) c;
		writeLines(new String(ch));
	}


 	public void write(String s,int off,int len) throws IOException {
		writeLines(s.substring(off,len));
    }

	String getBase64(String data) {
		byte utf8[];
		byte base64[];
		try {
			utf8 = data.getBytes("UTF8");
			base64 = Mime.encodeBase64(utf8);
			return new String(base64);
		}
		catch (Exception e) {
			System.out.println("Exception: never should come here " +
							   "(UTF8 converter)");
		}
		return data;
	}
	
	void writeLines(String block) {
		int index = 0;
		int n;
		if (currentLine != null)
			block = currentLine + block;

		while ((n = block.indexOf("\n", index)) != -1) {
			String line;
			int pos;
			String newline = null;
			line = block.substring(index, n);

			if (line.startsWith("#") == false) {
				String data;
				byte base64[];
				String s;
				// change ":" to "::" for data needs to be base64'ed
				if ((pos = line.indexOf("::")) != -1) {
					data = line.substring(pos + 2, line.length());
					if (Mime.isSafe(data) == false) {
						newline = line.substring(0, pos + 2) + " " +
							getBase64(data.trim());
					}
				}
				else if ((pos = line.indexOf(":")) != -1) {
					data = line.substring(pos + 1, line.length());
					if (Mime.isSafe(data) == false) {
						newline = line.substring(0, pos + 1) + ": " +
							getBase64(data.trim());
					}
				}
			}

			try {
				if (newline != null) {
					int i, len;
					for (i = 0; i < newline.length(); i += 76) {
						len = newline.length() - i;
						if (i > 0) {
							out.newLine();
							out.write(' ');						
						}
						if (len > 76)
							out.write(newline, i, 76);
						else						
							out.write(newline, i, len);
					}
					
				} else {
					out.write(line, 0, line.length());
				}
				out.newLine();
			}
			catch (Exception eb) {
				System.out.println("Exception:  BufferedWriter output");
			}

			index = n + 1;
		}
		currentLine = new String(block.substring(index, block.length()));
	}

	void writeCurrentLine() {
		String newline = null;
		String line;
		int pos;

		if (currentLine != null) {
			line = currentLine;
			if (line.startsWith("#") == false) {
				String data;
				byte base64[];
				String s;
				// change ":" to "::" for data needs to be base64'ed
				if ((pos = line.indexOf("::")) != -1) {
					data = line.substring(pos + 2, line.length());
					if (Mime.isSafe(data) == false) {
						newline = line.substring(0, pos + 2) + " " +
							getBase64(data.trim());
					}
				}
				else if ((pos = line.indexOf(":")) != -1) {
					data = line.substring(pos + 1, line.length());
					if (Mime.isSafe(data) == false) {
						newline = line.substring(0, pos + 1) + ": " +
							getBase64(data.trim());
					}
				}
			}

			try {
				if (newline != null) {
					int i, len;
					for (i = 0; i < newline.length(); i += 76) {						
						len = newline.length() - i;
						if (i > 0) {
							out.newLine();
							out.write(' ');						
						}
						if (len > 76)
							out.write(newline, i, 76);
						else						
							out.write(newline, i, len);
					}
					
				}
				else {
					out.write(line, 0, line.length());
				}
			}
			catch (Exception eb) {
				System.out.println("Exception:  BufferedWriter output");
			}
		}
	
		currentLine = null;
	}
}
