/**
 * Reader for escaped unicode
 */
import java.io.*;

class PropertyBufferedReader extends BufferedReader {
	private BufferedReader in;
	boolean g_finishReading = false;
	String leftOver = null;

    public PropertyBufferedReader(BufferedReader in) {
		super(in);
		this.in = in;
	}

    public PropertyBufferedReader(BufferedReader in, int size) {
		super(in, size);
		this.in = in;
    }

	public int read(char cbuf[],int off,int len) throws IOException {
		String block;
		String line;


		if (g_finishReading == true)
			return -1;

		line = readLine();
		if (line == null) {
			g_finishReading = true;
			return -1;
		}

		block = line + "\n";
		while (true) {
			try {
				line = readLine();
				if (line == null) {
					g_finishReading = true;
					break;
				}
			}
			catch(Exception e) {
				g_finishReading = true;
				break;
			}

			block += line + "\n";

			if (block.length() >= (off + len))
				break;
		}

		if (block.length() > (off + len)) {
			leftOver =
				new String(block.substring(off + len, block.length()));
			block.getChars(off, off + len, cbuf, 0);
			return len;
		}
		else  {
			leftOver = null;
			block.getChars(off, block.length(), cbuf, 0);
			return block.length() - off;
		}
   	}

 	public int read() throws IOException {
		char ch;
		if (leftOver != null) {
			ch = leftOver.charAt(0);
			leftOver = leftOver.substring(1, leftOver.length());
			return ch;
		}

		return in.read();
	}

	public String readLine() throws IOException {
		synchronized (lock) {
			// if space at beginning, keep reading
			// if end of file, return current buffer

			String line;
			
			line = in.readLine();
			if (line.indexOf("\\u") != -1) {
				byte[] bytes;
				bytes = line.getBytes();
				line = Mime.escapedUnicodeToText(bytes);
			}

			return line;
		}
	}
}
