/**
 * Writer for escaped unicode
 */
import java.io.*;

class PropertyBufferedWriter extends BufferedWriter {
	private BufferedWriter out;
	private String currentLine = null;

	public PropertyBufferedWriter(BufferedWriter out) {
		super(out);
		this.out = out;
	}

	public PropertyBufferedWriter(BufferedWriter out, int sz) {
		super(out, sz);
		this.out = out;
	}

	public void close() throws IOException {
		writeCurrentLine();
		out.close();
	}

	public void flush() throws IOException {
		writeCurrentLine();
		out.flush();
	}

	public void newLine() throws IOException {
		writeCurrentLine();
		out.newLine();
	}

	public void write(char cbuf[],int off,int len) throws IOException {
		writeLines(new String(cbuf, off, len));
    }

	public void write(int c) throws IOException {
		char ch[];

		ch = new char[1];
		ch[0] = (char) c;
		writeLines(new String(ch));
	}


 	public void write(String s,int off,int len) throws IOException {
		writeLines(s.substring(off,len));
    }

	String getBase64(String data) {
		byte utf8[];
		byte base64[];
		try {
			utf8 = data.getBytes("UTF8");
			base64 = Mime.encodeBase64(utf8);
			return new String(base64);
		}
		catch (Exception e) {
			System.out.println("Exception: never should come here " +
							   "(UTF8 converter)");
		}
		return data;
	}

	void writeLines(String block) {
		int index = 0;
		int n;
		if (currentLine != null)
			block = currentLine + block;

		while ((n = block.indexOf("\n", index)) != -1) {
			String line;
			int pos;
			String newline = null;
			line = block.substring(index, n);

			line = Mime.textToEscapedUnicode(line);

			try {
				out.write(line, 0, line.length());
				out.newLine();
			}
			catch (Exception eb) {
				System.out.println("Exception:  BufferedWriter output");
			}

			index = n + 1;
		}
		currentLine = new String(block.substring(index, block.length()));
	}

	void writeCurrentLine() {
		String newline;

		if (currentLine != null) {
			newline = Mime.textToEscapedUnicode(currentLine);
			try {
				out.write(newline, 0, newline.length());
			}
			catch (Exception e) {
				System.out.println("Prolem to write last line out");
			}
			currentLine = null;
		}
	}
}
