/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import netscape.ldap.*;
import netscape.ldap.util.DN;

/**
 * OrgChart
 * Test driver for ManagementParser and OrgChartPanel -
 * gets a reporting hierarchy from a directory and displays
 * an Org Chart panel in a dialog.
 *
 * @version 1.0
 * @author rweltman
 **/
public class OrgChart implements ActionListener {
    public OrgChart( String[] args ) {
        // Parse arguments
        String host = "localhost";
        int port = 389;
        String base = null;
        String rootDN = null;
        String authDN = null;
        String authPassword = null;
        boolean invalid = false;
        int i = 0;
        while ( i < args.length ) {
            if ( args[i].compareTo("-b") == 0 ) {
                if ( (i + 1) < args.length ) {
                    base = args[i+1];
                    i++;
                }
            } else if (args[i].compareTo("-D" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    authDN = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-w" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    authPassword = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-h" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    host = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-p" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    port = Integer.parseInt(args[i+1]);
                    i++;
                }
            } else if ( args[i].compareTo("-r" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    rootDN = args[i+1];
                    i++;
                }
            } else {
                invalid = true;
            }
            i++;
        }

        if ( invalid || (base == null) ) {
            System.err.println(
                "Usage: OrgChart -b BASEDN " +
                "[-D AUTHDN] [-w AUTHPASSWORD] " +
                "[-r ROOTMANAGERDN] [-h HOST] [-p PORT]" );
            System.exit( 1 );
        }

        // Connect to server
        _ldc = new LDAPConnection();
        try {
            _ldc.connect( host,
                          port,
                          authDN,
                          authPassword );
        } catch ( LDAPException e ) {
            System.err.println( e );
            System.exit( 1 );
        }

        // Get a tree of users, organized by reporting structure
        DefaultMutableTreeNode root = null;
        try {
            root = ManagementParser.parseTree( _ldc, base, rootDN );
        } catch ( LDAPException e ) {
            System.err.println( e );
            System.exit( 1 );
        }

        // Create a panel component with the nodes found
        OrgChartPanel org = new OrgChartPanel( root );
        org.setSubordinateTextColor( Color.blue );
        org.addActionListener( this );

        // Prepare the dialog/frame content panel
        Dimension dim = new Dimension( 600, 450 );
        new SimpleDialog( "OrgChart " + base, org, 0, 0,
                          dim, true ).show();

        if ( _debug ) {
            ManagementParser.printTree( root );
        }
    }

    /**
     * Handle events from clicking on a user node
     *
     * @param e Event from a button; the action command
     * is the DN of the user entry
     */
    public void actionPerformed( ActionEvent e ) {
        if ( e.getSource() instanceof JButton ) {
            String dn = e.getActionCommand();
            System.out.println( dn );
            // The action command is the DN of an entry to view
            JTable table = new PropertyTable( _ldc, dn );
            Dimension dim = new Dimension( 500, 300 );
            new SimpleDialog( dn, table, 20, 20,
                              dim, false ).show();
        }
    }

    static public void main ( String[] args ) {
        new OrgChart( args );
    }

    static private final boolean _debug = false;
    private LDAPConnection _ldc = null;
}
