
// Author: Peter English english@quiknet.com
// Used by permission

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;


/*
    Prefetches the blob data into a vector of 4KB buffers
    to determine data size

*/


class CBlobInputStream extends InputStream {

    ServletInputStream m_inStream;
    Vector m_vctBuffer;
    String m_szBoundary;
    CParseRFC1867 m_parseRFC1867;

    byte[] m_byteBuffer;

    int m_nTotalBytes;

    int m_nTotalBytesRead;
    int m_nBytesRead;
    int m_nCurIndex;
    boolean m_bEof;

    int m_nPreFetchSegmentSize;
    int m_nPreFetchBytesRead;
    int m_nPreFetchCurIndex;
    int m_nPreFetchStartIndex;
    boolean m_bPreFetchHasCrLf;
    boolean m_bPreFetchEof;

    public CBlobInputStream(ServletInputStream inStream, String szBoundary, CParseRFC1867 parseRFC1867) {

        m_inStream = inStream;
        m_szBoundary = szBoundary;
        m_parseRFC1867 = parseRFC1867;

        m_nTotalBytes = 0;

        m_nPreFetchSegmentSize = 4096;
        m_nPreFetchBytesRead = 0;
        m_nPreFetchStartIndex = 0;
        m_nPreFetchCurIndex = m_nPreFetchSegmentSize;
        m_bPreFetchHasCrLf = false;
        m_bPreFetchEof = false;
        m_vctBuffer = new Vector();

        m_nTotalBytesRead = 0;
        m_nBytesRead = 0;
        m_nCurIndex = m_nPreFetchSegmentSize;
        m_bEof = false;

        m_byteBuffer = new byte[m_nPreFetchSegmentSize];
    }

    public void preFetch() {
        while (!m_bPreFetchEof) {

            int nSegmentRead = 0;
            int nBytesRead = 0;

            byte[] bytePreFetchBuffer = new byte[m_nPreFetchSegmentSize];

            while (!m_bPreFetchEof && m_nPreFetchSegmentSize - nSegmentRead > 0) {

                try {
                    nBytesRead = preFetchRead(bytePreFetchBuffer, nSegmentRead, m_nPreFetchSegmentSize - nSegmentRead);
                }
                catch (IOException e) {
                    nBytesRead = -1;
                }

                if (nBytesRead != -1) {
                    nSegmentRead += nBytesRead;
                    m_nTotalBytes += nBytesRead;
                }
            }

            if (nSegmentRead > 0) {
                m_vctBuffer.addElement(bytePreFetchBuffer);
            }
        }
    }


    public int available() throws IOException {
        return m_nBytesRead > 0 ? m_nBytesRead - m_nCurIndex : 0;
    }

    protected void getNextSegment() {
        if (m_vctBuffer.size() > 0) {
            m_nCurIndex = 0;
            m_byteBuffer = (byte[]) m_vctBuffer.firstElement();
            m_vctBuffer.removeElement(m_byteBuffer);
            m_nBytesRead = (m_vctBuffer.size() > 0) ? m_nPreFetchSegmentSize : (m_nTotalBytes % m_nPreFetchSegmentSize);
        }
        else {
            m_nBytesRead = -1;
        }
    }

    public int read() throws IOException {

        if (m_bEof) {
            return -1;
        }

        if (m_nCurIndex >= m_nBytesRead) {
            getNextSegment();
        }

        if (m_nBytesRead < 0) {
            m_bEof = true;
            return -1;
        }
        else {
            return m_byteBuffer[m_nCurIndex ++];
        }
    }

    public int read(byte[] b, int off, int len) throws IOException {

        if (m_bEof) {
            return -1;
        }

        if (m_nCurIndex >= m_nBytesRead) {
            getNextSegment();
        }

        if (m_nBytesRead < 0) {
            m_bEof = true;
            return -1;
        }
        else {

            int nBytesCopied = Math.min(len, m_nBytesRead - m_nCurIndex);

            System.arraycopy(m_byteBuffer, m_nCurIndex, b, off, nBytesCopied);
            m_nCurIndex += nBytesCopied;
            return nBytesCopied;
        }
    }

    public int getTotalBytes() {
        return m_nTotalBytes;
    }

    public boolean readEOF() {
        return m_bEof;
    }

    protected String readLine() {

        try {
            m_nPreFetchStartIndex = 2;

            //Begin Bug Fix.  Should be m_byteBuffer.length - 3
            //m_nPreFetchBytesRead = m_inStream.readLine(m_byteBuffer,  m_nPreFetchStartIndex, m_byteBuffer.length - 1);
            //End Bug Fix

            m_nPreFetchBytesRead = m_inStream.readLine(m_byteBuffer,  m_nPreFetchStartIndex, m_byteBuffer.length - 3);
            
            // Check if last character is a carriage return
            // If it is, read another character.
            // We are doing this to detect the \r\n character combination
            // Note: in the readLine above, we left room in the buffer for this potential character read.
            
            if (m_byteBuffer[m_nPreFetchBytesRead +  m_nPreFetchStartIndex - 1] == '\r') {
                int nRead = m_inStream.readLine(m_byteBuffer,  m_nPreFetchStartIndex + m_nPreFetchBytesRead, 1);
                m_nPreFetchBytesRead += (nRead > 0 ? nRead : 0);
            }

            if (m_nPreFetchBytesRead != -1) {
                m_nPreFetchCurIndex =  m_nPreFetchStartIndex;


                boolean bHasCrLf = (m_nPreFetchBytesRead > 1 && m_byteBuffer[m_nPreFetchStartIndex + m_nPreFetchBytesRead - 2] == '\r' && m_byteBuffer[m_nPreFetchStartIndex + m_nPreFetchBytesRead - 1] == '\n');

                String szLine = null;
                boolean bEndOfData = false;
                
                // If the line read starts with --Some delimeter string..., etc.
                // then this is the end of the data for this blob
                
                if (m_nPreFetchBytesRead > 1 && m_byteBuffer[m_nPreFetchStartIndex] == '-' && m_byteBuffer[m_nPreFetchStartIndex + 1] == '-') {
                    szLine = new String(m_byteBuffer,  m_nPreFetchStartIndex, m_nPreFetchBytesRead);
                    bEndOfData = szLine.startsWith(m_szBoundary);
                }

                // If the end of blob data is not reached and the prevoius call to this method
                // read in a \r\n, that \r\n was deferred to this call to this method.
                // If this is the case, the append \r\n to the beginnning of the read buuffer.
                // Note that space was reserved in the buffer for these characters.
                
                // Note that if readLine resulted in \r\n at the end of the line, the \r\n is deferred
                // to the next call of readLine.  This simplifies chacking if the \r\n is part of the 
                // blob data or if it is part the end og blob data without having to prefetch an extra line.
                
                if (!bEndOfData) {
                    if (m_bPreFetchHasCrLf) {
                        m_nPreFetchBytesRead += 2;
                        m_nPreFetchStartIndex = 0;
                        m_nPreFetchCurIndex = 0;
                        m_byteBuffer[0] = (byte)'\r';
                        m_byteBuffer[1] = (byte)'\n';
                    }
                }
                else {

                    m_parseRFC1867.checkPopBoundary(szLine);

                    m_nPreFetchBytesRead = -1;
                    return null;
                }

                m_bPreFetchHasCrLf = bHasCrLf;

                if (m_bPreFetchHasCrLf) {
                    m_nPreFetchBytesRead -= 2;
                }

                return null;

            }
        }
        catch (IOException e) {};

        return null;
    }

    protected int preFetchRead(byte[] b, int off, int len) throws IOException {

        if (m_bPreFetchEof) {
            return -1;
        }

        if (m_nPreFetchCurIndex >= m_nPreFetchStartIndex + m_nPreFetchBytesRead) {
            readLine();
        }

        if (m_nPreFetchBytesRead < 0) {
            m_bPreFetchEof = true;
            return -1;
        }
        else {

            int nBytesCopied = Math.min(len, (m_nPreFetchBytesRead + m_nPreFetchStartIndex) - m_nPreFetchCurIndex);

            System.arraycopy(m_byteBuffer, m_nPreFetchCurIndex, b, off, nBytesCopied);
            m_nPreFetchCurIndex += nBytesCopied;
            return nBytesCopied;
        }
    }


}


class CParseRFC1867 {

    static final int PARAMETER = 0;
    static final int FILE = 1;

    ServletInputStream m_inStream = null;
    Stack m_stackBoundary = new Stack();
    String m_szBoundary = null;
    byte m_byteBuffer[] = new byte[4096];
    int m_nParameterType = -1;
    boolean m_bHasMore = true;


    String m_szParameterName;
    String m_szParameterValue;
    String m_szFileName;

    public CParseRFC1867(HttpServletRequest request) {

        try {
            m_inStream = request.getInputStream();
        }
        catch (IOException e) {};

        String szContentType = request.getContentType();

        getBoundary("content-type: " + szContentType);
        m_stackBoundary.push(m_szBoundary);

        readToNextBoundary();

}

    protected boolean getBoundary(String szLine) {
        if (szLine != null) {

            String szLineLowerCase = szLine.toLowerCase();


            if (szLineLowerCase.indexOf("content-type") != -1
                && szLineLowerCase.indexOf("boundary")!= -1) {

                if (m_szBoundary != null) {
                    m_stackBoundary.push(m_szBoundary);
                }

                m_szBoundary = szLine.substring(szLineLowerCase.lastIndexOf("boundary") + 9);
                m_szBoundary = "--" + m_szBoundary;

                return true;
            }
        }

        return false;
    }

    public void checkPopBoundary(String szLine) {
        if (szLine.startsWith(m_szBoundary + "--")) {
            m_szBoundary = (String) m_stackBoundary.pop();
        }
    }

    public boolean hasMoreContent() {



        return m_bHasMore && !m_stackBoundary.empty();
    }

    public int getContentType() {
        return m_nParameterType;
    }

    public String getParameterName() {
        return m_szParameterName;
    }

    public String getParameterValue() {
        return m_szParameterValue;
    }

    public String getFileName() {
        return m_szFileName;
    }

    public CBlobInputStream getBlobInputStream() {
        return new CBlobInputStream(m_inStream, m_szBoundary, this);
    }

    protected String readLine() {

        try {
            int nBytesRead = m_inStream.readLine(m_byteBuffer, 0, m_byteBuffer.length);

            if (nBytesRead != -1) {
                return new String(m_byteBuffer, 0, nBytesRead);
            }
        }
        catch (IOException e) {};

        m_bHasMore = false;

        return null;
    }

    public void readToNextBoundary() {
        String szLine = null;
        while ((szLine = readLine()) != null) {

            if (szLine.startsWith(m_szBoundary)) {

                checkPopBoundary(szLine);

                return;
            }
        }
    }

    public void getContent() {

        String szLine = readLine();

        if (szLine != null) {

            String szLineLowerCase = szLine.toLowerCase();

            if (szLineLowerCase.indexOf("content-disposition") != -1) {

                int nNameIndex = szLineLowerCase.indexOf("name=");
                if (nNameIndex != -1) {

                    m_nParameterType = PARAMETER;

                    int nLastNameIndex = szLineLowerCase.indexOf('"', nNameIndex + 6);
                    m_szParameterName = szLine.substring(nNameIndex + 6, nLastNameIndex);

                    do {

                        int nFileNameIndex = szLineLowerCase.indexOf("filename=");
                        if (nFileNameIndex != -1) {

                            m_nParameterType = FILE;

                            int nLastFileNameIndex = szLineLowerCase.indexOf('"', nFileNameIndex + 10);
                            m_szFileName = szLine.substring(nFileNameIndex + 10, nLastFileNameIndex);
                            if (m_szFileName.lastIndexOf('\\') != -1) {
                                m_szFileName = m_szFileName.substring(m_szFileName.lastIndexOf('\\') + 1);
                            }

                            if (m_szFileName.lastIndexOf('/') != -1) {
                                m_szFileName = m_szFileName.substring(m_szFileName.lastIndexOf('/') + 1);
                            }
                        }
                        szLine = readLine();
                        if (getBoundary(szLine)) {
                            readToNextBoundary();
                            szLine = readLine();
                        }
                        szLineLowerCase = szLine.toLowerCase();


                    } while (!szLine.equals("\r\n"));

                    if (m_nParameterType == PARAMETER) {

                        szLine = readLine();
                        while (!szLine.startsWith(m_szBoundary)) {

                            m_szParameterValue = szLine.substring(0, szLine.length() - 2);
                            szLine = readLine();
                        }
                        checkPopBoundary(szLine);
                    }

                }
            }
        }
    }

}
