/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */
import netscape.ldap.*;
import java.util.*;

/**
 * Do a parallel, multiplexed search to multiple servers
 * 
 * @version 1.0
 * @author rweltman
 **/
public class MultiplexedSearch {
    public static void main( String[] args ) {
        String MY_FILTER = "objectclass=*";
        // Three hosts, three ports, three search bases
        String[] HOSTS =
            { "localhost", "ldap.acme.com", "ds.airius.com" };
        int[] PORTS = { 389, 24000, 32000 };
        String[] BASES =
            { "o=Airius.com", "o=acme.com", "dc=airius,dc=com" };
        LDAPConnection[] lds = new LDAPConnection[HOSTS.length];
        try {
            for( int i = 0; i < lds.length; i++ ) {
                lds[i] = new LDAPConnection();
                lds[i].connect( HOSTS[i], PORTS[i] );
                System.out.println( "Connected to " + HOSTS[i] + ':' + PORTS[i] );
            }
            // Get a response listener for one search
            LDAPSearchListener listener =
                lds[0].search( BASES[0],
                               lds[0].SCOPE_SUB,
                               MY_FILTER,
                               null,
                               false,
                               (LDAPSearchListener)null );
            // Share the listener
            for( int i = 1; i < lds.length; i++ ) {
                lds[i].search( BASES[i],
                               lds[i].SCOPE_SUB,
                               MY_FILTER,
                               null,
                               false,
                               listener );
            }

            // Loop on results until finished
            // This is the same as in the previous example
            LDAPMessage msg;
            while( (msg = listener.getResponse()) != null ) {
                // Next directory entry
                if ( msg instanceof LDAPSearchResultReference ) {
                    String[] urls =
                        ((LDAPSearchResultReference)msg).getUrls();
                    // Do something with the referrals...
                } else if ( msg instanceof LDAPSearchResult ) {
                    LDAPEntry entry =
                        ((LDAPSearchResult)msg).getEntry();
                    // The rest of the processing is the same as for
                    // a synchronous search
                    System.out.println( entry.getDN() );
                } else {
                    // A search response
                    LDAPResponse res = (LDAPResponse)msg;
                    int status = res.getResultCode();
                    String err =
                        LDAPException.errorCodeToString( status );
                    System.out.println( "Result of " + res.getID() + ": " +
                                        err );
                    if ( status == LDAPException.SUCCESS ) {
                        // Nothing to do
                    } else {
                        throw new LDAPException( err,
                                                 status,
                                                 res.getErrorMessage(),
                                                 res.getMatchedDN() );
                    }
                }
            }
        } catch( LDAPException e ) {
            System.out.println( "Error: " + e.toString() );
        }

        // Done, so disconnect
        for( int i = 1; i < lds.length; i++ ) {
            if ( (lds[i] != null) && lds[i].isConnected() ) {
                try {
                    lds[i].disconnect();
                } catch ( LDAPException e ) {
                    System.out.println( e.toString() );
                }
            }
        }
        System.exit( 0 );
    }
}
