/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import netscape.ldap.*;
import netscape.ldap.util.*;
import java.util.*;

// Class to experiment with filter configuration files

public class CreateFilter {

    /**
     * Expand a filter expression using a filter configuration
     * file
     *
     * @param args filterfile, filter
     */
    public static void main( String[] args ) {

        if ( args.length != 2 ) {
            System.out.println( "Usage: java CreateFilter " +
                                "<filterfile> <search " +
                                "expression> ");
            System.out.println( "Example:" );
            System.out.println( "    java CreateFilter " +
                                "tryfilt.conf \"*peter*\"" );
            System.exit( 1 );
        }

        LDAPFilterDescriptor filterDesc = null;
        LDAPFilterList filtlist = null;
        String srchfilter = "";
        int numfilts = 0;

        try {
            filterDesc = new LDAPFilterDescriptor( args[0] );
        } catch (Exception e) {
            System.out.println( "Cannot load file: " +
                                args[0] );
            System.exit(0);
        }

        try {
            filtlist =
                filterDesc.getFilters( "search", args[1] );
            numfilts = filtlist.numFilters();
            if ( numfilts > 1 ) {
                srchfilter += "(|";
            }
            while ( filtlist.hasMoreElements() ) {
                LDAPFilter fline = filtlist.next();
                // The filter is next
                String fstr = fline.getFilter();
                System.out.println( "Filter = " + fstr );
                srchfilter += fstr;
            }
            if ( numfilts > 1 ) {
                srchfilter += ")";
            }
        } catch ( Exception e ) {
            System.out.println( "Filter error: " +
                                e.toString() );
        }
        System.out.println( "Search filter = " +
                            srchfilter );
    }

}
