/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */


import netscape.ldap.*;
import java.util.*;

/**
 * Read an entry from the directory and display the contents
 */

public class EntryRead {

    /**
     * Read an entry from the directory and display the contents
     *
     * @param args host, port, authDN, password, dn
     */
    public static void main( String[] args ) {

        if ( args.length != 5 ) {
            System.out.println( "Usage: java EntryRead " +
                                "<host> <port> " +
                                "<authdn> <password> " +
                                "<dn>" );
            System.out.println( "Example:" );
            System.out.println( "    java EntryRead " +
                                "localhost 389 " +
                                "\"\" \"\" " +
                                "\"uid=scarter, ou=people, " +
                                "o=airius.com\"" );
            System.exit(1);
        }

        String host = args[0];
        int port = Integer.parseInt( args[1] );
        String authid = args[2];
        String authpw = args[3];
        String base = args[4];
        String[] ATTRS = { "cn","mail","telephonenumber" };

        int status = -1;
        LDAPConnection ld = new LDAPConnection();
        try {
            // Connect to server and authenticate
            ld.connect( host, port, authid, authpw );

            LDAPEntry entry = ld.read( base, ATTRS );
            prettyPrint( entry, ATTRS );
            status = 0;
        } catch ( LDAPReferralException e ) {
            // Ignore referrals
        } catch ( LDAPException e ) {
            System.out.println( e.toString() );
        }

        // Done, so disconnect
        if ( (ld != null) && ld.isConnected() ) {
	    try {
                ld.disconnect();
            } catch ( LDAPException e ) {
                System.out.println( e.toString() );
            }
        }
        System.exit( status );
    }

    /**
     * Print names and values of attributes in an entry
     *
     * @param entry Entry containing attributes
     * @param attrs Array of attribute names to display
     */
    public static void prettyPrint( LDAPEntry entry, 
                                    String[] attrs ) {
        System.out.println( "DN: " + entry.getDN() );   

        // Use array to pick attributes. We could have
        // enumerated them all using LDAPEntry.getAttributes
        // but this gives us control of the display order.
        for ( int i = 0; i < attrs.length; i++ ) {
            LDAPAttribute attr =
                entry.getAttribute( attrs[i] );
            if ( attr == null ) {
                System.out.println( attrs[i] +
                                    " not present" );
                continue;
            }
            Enumeration enumVals = attr.getStringValues();
            // Enumerate on values for this attribute
            boolean hasVals = false;
            while ( (enumVals != null) &&
                    enumVals.hasMoreElements() ) {
                String val = (String)enumVals.nextElement();
                System.out.println( attrs[i] + ": " + val );
                hasVals = true;
            }
            if ( !hasVals ) {
                System.out.println( attrs[i] +
                                    " has no values" );
            }
        }
        System.out.println("-------------------");
    }
}
