/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.util.*;
import netscape.ldap.*;

/**
 * Class to rename an entry by changing its RDN
 */
public class RenameRDN {

    /**
     * Rename an entry by changing its RDN
     *
     * @param args host, port, dn, newrdn, [newparentdn, ]
     * authdn, password, Y|N
     */
    public static void main( String[] args ) {

        if ( (args.length != 7) &&
             (args.length != 8) ) {
            System.out.println( "Usage: java RenameRDN " +
                                "<host> <port> " +
                                "<dn> <newRDN> " +
                                "<[newParentDN>] " +
                                "<authdn> <password> " +
                                "Y|N" );
            System.out.println( "Example:" );
            System.out.println( "    java RenameRDN " +
                                "localhost 389 " +
                                "\"ou=old," +
                                "o=airius.com\" " +
                                "\"ou=new\" " +
                                "\"cn=Directory Manager\" " +
                                "password Y" );
            System.out.println( "Y|N is for deleting the " +
                                "old RDN or not (and leave " +
                                "it in the entry)" );
            System.exit(1);
		}

        String host = args[0];
        int port = Integer.parseInt( args[1] );
		String oldDN = args[2];
		String newRDN = args[3];
        String authid;
        String authpw;
        String newParentDN = null;
        if ( args.length == 7 ) {
            authid = args[4];
            authpw = args[5];
        } else {
            newParentDN = args[4];
            authid = args[5];
            authpw = args[6];
        }

      
        boolean  deleteOld = false;
        if ( args[args.length-1].equals("Y") ) {
            deleteOld = true;
        }

        LDAPConnection ld = new LDAPConnection();
        try {
            // Connect to server and authenticate
            ld.connect( host, port, authid, authpw );
            System.out.println( "Connected and " +
                                "authenticated " +
                                "to " + host );
            ld.read( oldDN );
            if ( newParentDN == null ) {
                System.out.println( "Executing form 1 of " +
                                    "rename");
                ld.rename( oldDN, newRDN, deleteOld );
                System.out.println( "Renamed " + oldDN +
                                    " to " + newRDN );
            } else {
                System.out.println( "Executing form 2 of " +
                                    "rename");
                ld.rename( oldDN, newRDN, newParentDN,
                           deleteOld );
                System.out.println( "Renamed " + oldDN +
                                    " to " + newParentDN );
            }

        } catch( LDAPException e ) {
            System.out.println( e.toString() );
        }

        // Done, so disconnect
        if ( (ld != null) && ld.isConnected() ) {
            try {
                ld.disconnect();
            } catch ( LDAPException e ) {
                System.out.println( e.toString() );
            }
        }
        System.exit(0);
    }
}



