/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import netscape.ldap.*;
import java.util.*;

/**
 * Given a uid and password, find the corresponding entry
 * and authenticate
 */

public class UidLogin {

    /** 
     * Search for the uid and then authenticate with the
     * password
     *
     * @param args host, port, uid, password
     */
     public static void main( String[] args ) {

        if ( args.length != 5 ) {
            System.out.println( "Usage: java UidLogin " +
                                "<host> <port> <baseDN> " +
                                "<uid> <password>" );
            System.out.println( "Example:" );
            System.out.println( "    java UidLogin " +
                                "localhost 389 " +
                                "\"o=airius.com\" " +
                                "cathyp password" );
            System.exit(1);
        }

        String host = args[0];
        int port = Integer.parseInt( args[1] );
        String baseDN = args[2];
        String authid = args[3];
        String authpw = args[4];

        if ( (authid.length() == 0) ||
             (authpw.length() == 0) ||
             (authid.indexOf('*') >= 0) ) {
            System.out.println( "You must supply a uid and " +
                                "password" );
            System.exit(1);
        }

        boolean authed = false;
        LDAPConnection ld = new LDAPConnection();
        try {
            // Connect to server anonymously
            ld.connect( host, port );

            // Search for entries with a matching uid
            LDAPSearchConstraints cons =
                ld.getSearchConstraints();
            cons.setBatchSize( 0 );
            LDAPSearchResults results = 
                ld.search( baseDN,
                           ld.SCOPE_SUB,
                           "uid=" + authid,
                           new String[] {ld.NO_ATTRS},
                           false,
                           cons );
            // There should be exactly one match
            if ( !results.hasMoreElements() ) {
                System.out.println( "uid not found" );
            } else if ( results.getCount() > 1 ) {
                System.out.println( "More than one matching " +
                                    "uid" );
            } else {
                LDAPEntry entry = results.next();
                String authDN = entry.getDN();
                System.out.println( "uid maps to: " +
                                    authDN );
                ld.authenticate( authDN, authpw );
                authed = true;
            }
        } catch( LDAPException e ) {
            System.out.println( e.toString() );
        }

        // Done, so disconnect
        if ( (ld != null) && ld.isConnected() ) {
            try {
                ld.disconnect();
            } catch ( LDAPException e ) {
                System.out.println( e.toString() );
            }
        }

        int rc = 0;
        if ( authed ) {
            System.out.println( "Authenticated!" );
        } else {
            System.out.println("Not Authenticated!");
            rc = 1;
        }
        System.exit(rc);
     }
}
