/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.io.*;
import java.util.*;
import java.awt.*;
import java.applet.*;
import netscape.ldap.*;
// Must have Navigator's java40.jar in the CLASSPATH
import netscape.security.PrivilegeManager;
// Must have Internet Explorer's classes.zip in the CLASSPATH
import com.ms.security.*;

/**
 * Invisible applet that searches a directory and returns a
 * single string with all results, formatted with line feeds.
 * The host, port, and base (base DN for search) are read from
 * applet parameter tags.
 */
public class DirApplet extends Applet {
    /**
     * Standard applet entry point
     */
    public void init() {
        super.init();
        // Get parameters from applet tags if present 
        String s = getParameter( "host" );
        if ( (s != null) && (s.length() > 0) ) {
            host = s;
        }
        s = getParameter( "base" );
        if ( (s != null) && (s.length() > 0) ) {
            base = s;
        }
        s = getParameter( "port" );
        if ( (s != null) && (s.length() > 0) ) {
            port = Integer.parseInt( s );
        }
        // Detect the browser version. 
        try {
            String vendor =
                System.getProperty("java.vendor");
            if (vendor.indexOf("Netscape") != -1) {
                browser = NETSCAPE;
            } else if (vendor.indexOf("Microsoft")
                != -1) {
                browser = MICROSOFT;
            }
        } catch (SecurityException e) {
            System.out.println(e.toString());
        }
    }

    /**
     * Searches the directory and returns the
     * results as a single string. Entries, 
     * attributes, and values are delimited 
     * by newline characters.
     *
     * @param filter An LDAP search filter expression
     * @return A string containing all search results, or
     * an error message with stack trace on error
     */
    public String search(String filter) {
        String str = new String();
        try {
            // Get permission to connect to the network
            if ( browser == NETSCAPE ) {
                PrivilegeManager.enablePrivilege(
                    "UniversalConnect" );
                PrivilegeManager.enablePrivilege(
                    "UniversalThreadAccess" );
            } else if ( browser == MICROSOFT ) {
                PolicyEngine.assertPermission(
                    PermissionID.NETIO );
            }
            // Connect to the LDAP server. 
            connection = new LDAPConnection();
            connection.connect( host, port );
            // Perform the search
            LDAPSearchResults results
                = connection.search( base,
                                     LDAPConnection.SCOPE_SUB,
                                     filter, null, false );
            // Revert the privileges so that
            // they are no longer enabled. 
            if ( browser == NETSCAPE ) {
                PrivilegeManager.revertPrivilege(
                    "UniversalConnect" );
                PrivilegeManager.revertPrivilege(
                    "UniversalThreadAccess" );
            } else if ( browser == MICROSOFT ) {
                PolicyEngine.revertPermission(
                    PermissionID.NETIO );
            }
            str = "Entries:\n";
            // Loop through the results. 
            while ( results.hasMoreElements() ) {
                LDAPEntry entry = results.next();
                // Start each result with the DN. 
                str += entry.getDN() + "\n";
                // Get the attributes for the entry. 
                LDAPAttributeSet attributeSet =
                    entry.getAttributeSet();
                Enumeration enumAttrs =
                    attributeSet.getAttributes();
                str += "Attributes:\n";
                // Delimit the attribute names
                // and values with newline characters.
                while ( enumAttrs.hasMoreElements() ) {
                    LDAPAttribute anAttr =
                        (LDAPAttribute)enumAttrs.nextElement();
                    String attrName = anAttr.getName();
                    str += "  " + attrName + "\n";
                    Enumeration enumVals =
                        anAttr.getStringValues();
                    // Get the values of the attribute. 
                    while ( enumVals.hasMoreElements() ) {
                        String aVal =
                            (String)enumVals.nextElement();
                        str += "    " + aVal + "\n";
                    }
                }
                str += "\n";
            }
        } catch ( Exception e ) {
            PrintWriter pWriter =
                new PrintWriter( new CharArrayWriter() );
            e.printStackTrace( pWriter );
            str += e.toString() + ":\n\t"
                + pWriter.toString();
            return str;
        }
        return str;
    }

    // Connection to directory 
    private LDAPConnection connection = null;
    // Default parameters for connection and search base
    private String host = "localhost";
    private int port = 389;
    private String base = "o=airius.com";
    // Browser type 
    private static final int NETSCAPE = 0;
    private static final int MICROSOFT = 1;
    private int browser = -1;
}
