/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */
import netscape.ldap.*;

/**
 * JSLDAPSearchResults.java<P>
 * JAVA class for Java Script to access LDAP server, which
 * wraps the Netscape Directory SDK for Java LDAPSearchResults class
 * to trap exceptions and provide some additional aid for Java/JS
 * transitions.
 */

public class JSLDAPSearchResults {
    public JSLDAPSearchResults( LDAPSearchResults res ) {
        super();
        _results = res;
    }

    /**
     * Returns the next LDAP entry from the search results
     * or <CODE>null</CODE> if the next result is an
     * exception (including a referral). If <CODE>null</CODE>,
     * the LDAP result code is available with getErrorCode.
     * @return The next LDAP entry in the search results.
     */
    public LDAPEntry next() {
        try {
            LDAPEntry entry = _results.next();
            _errorCode = 0;
            return entry;
        } catch( LDAPException e ) {
            _errorCode = e.getLDAPResultCode();
            return null;
        }
    }

    /**
     * Returns <CODE>true</CODE> if there are more search results
     * to be returned. Use this method in conjunction with the
     * <CODE>nextEntry</CODE> method to iterate
     * through each entry in the results.
     * @return <CODE>true</CODE> if there are more search results
     */
    public boolean hasMoreElements() {
        return _results.hasMoreElements();
    }

    /**
     * Sorts the search results.
     *
     * @param compare Comparator used to determine the sort
     * order of the results.
     */
    public synchronized void sort(LDAPEntryComparator compare) {
        _results.sort( compare );
    }

    /**
     * Returns a count of the entries in the search results.
     * @return Count of entries found by the search.
     */
    public int getCount() {
        return _results.getCount();
    }

    /**
     * Get latest LDAP error code
     *
     * @return Latest LDAP error code
     */
    public int getErrorCode() {
        return _errorCode;
    }

    private int _errorCode = 0;
    private LDAPSearchResults _results = null;
}
