# Makefile for the LDAP Filter classes
#
# This packages the netscape.ldap.util filter classes with the
# regular expression classes from Oro, Inc.
#
# The procedure here assumes you have built the LDAP SDK distribution
# with ns/sun-java/classsrc/javaldap.mk (so the LDAP classes are in
# ns/dist/classes).
#
# All required source can be fetched as follows:
#  cvs co -r server3_directory_branch ns/netsite/ldap/java/ldapfilter
#
ZIP=zip

MCOM_ROOT=../../../..
BASEDIR := $(shell cd $(MCOM_ROOT); pwd)
# Destination for class files and packages
LDAP_CLASS_DEST=$(BASEDIR)/dist/classes
FILTER_DEST=$(BASEDIR)/dist/ldapfilt
FILTER_CLASS_DEST=$(FILTER_DEST)/classes

# Set up the CLASSPATH automatically,
# so it doesn't have to be done outside of the make. You still have to
# have ifc11.jar in your CLASSPATH to compile the DNPicker, though.
JDKCOMP := $(shell which javac)
TMP_ARCH := $(shell uname -s)
ifeq ($(TMP_ARCH), WINNT)
  JLIB:=$(JDKCOMP:%bin/javac.exe=%lib/classes.zip)
  JDKLIB:=$(subst \,/,$(JLIB))
  SEP=;
else
  JDKLIB:=$(JDKCOMP:%bin/javac=%lib/classes.zip)
  SEP=:
endif
CLASSPATH:=$(CLASSPATH)$(SEP)$(LDAP_CLASS_DEST)$(SEP)$(BASEDIR)/netsite/ldap/java/ldapfilter$(SEP)$(JDKLIB)

SRCDIR=netscape/ldap
EXAMPLESRCDIR=.
DISTDIR=$(MCOM_ROOT)/dist/ldapfilt
CLASSDIR=$(DISTDIR)/classes
CLASSPACKAGEDIR=$(MCOM_ROOT)/dist/packages
ORODIR=$(CLASSDIR)/com/oroinc/text/regex
PACKAGENAME=ldapfilt.zip
JARNAME=ldapfilt.jar
CLASSPACKAGE=$(CLASSPACKAGEDIR)/$(PACKAGENAME)
EXAMPLEDIR=$(MCOM_ROOT)/dist/examples/java/ldapfilt

ifdef JDKDIR
  JDKBIN=$(JDKDIR)/bin/
endif

ifndef JAVAC
  ifeq ($(DEBUG), 1)
    JAVAC=$(JDKBIN)javac -g -classpath "$(CLASSPATH)"
#    JAVAC=$(JDKBIN)javac -g
  else
    JAVAC=$(JDKBIN)javac -O -classpath "$(CLASSPATH)"
#    JAVAC=$(JDKBIN)javac -O
  endif
endif

all: classes examples

classes: $(DISTDIR) $(CLASSDIR) OROCLASSES FILTERCLASSES

examples: $(EXAMPLEDIR)/Test-AWT EXAMPLES

package: $(CLASSPACKAGEDIR)
ifeq ($(ARCH), WINNT)
	cd $(DISTDIR)/classes; zip -r ../../packages/$(JARNAME) netscape com
else
	cd $(DISTDIR)/classes; zip -r ../../packages/$(JARNAME) netscape/ldap/util/*Filt* com
endif
#	cd $(CLASSPACKAGEDIR); zip $(PACKAGENAME) $(JARNAME)
#	cd $(DISTDIR); zip -r ../packages/$(PACKAGENAME) examples
#	cd $(DISTDIR); zip -r ../packages/$(PACKAGENAME)

FILTERCLASSES:
	cd $(SRCDIR)/util; $(JAVAC) -d $(FILTER_CLASS_DEST) *.java

OROCLASSES: $(ORODIR)
	cp -p com/oroinc/text/regex/*.class $(ORODIR)

EXAMPLES:
	cp -p $(EXAMPLESRCDIR)/Test-AWT/*.java $(EXAMPLEDIR)/Test-AWT
	cp -p $(EXAMPLESRCDIR)/Test-AWT/*.html $(EXAMPLEDIR)/Test-AWT
	cp -p $(EXAMPLESRCDIR)/Test-AWT/*.conf $(EXAMPLEDIR)/Test-AWT
	cp -p $(EXAMPLESRCDIR)/Test-AWT/README $(EXAMPLEDIR)/Test-AWT

$(CLASSPACKAGEDIR):
	mkdir -p $@

$(DISTDIR):
	mkdir -p $@

$(CLASSDIR):
	mkdir -p $@

$(ORODIR):
	mkdir -p $@

$(EXAMPLEDIR):
	mkdir -p $@

$(EXAMPLEDIR)/Test-AWT:
	mkdir -p $@
